﻿using Lavalink4NET.Remora.Discord;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Cassette.Commands;
using Remora.Commands.Extensions;
using Remora.Discord.API.Abstractions.Gateway.Commands;
using Remora.Discord.Commands.Extensions;
using Remora.Discord.Commands.Services;
using Remora.Discord.Gateway;
using Remora.Discord.Hosting.Extensions;

namespace Cassette;

file class Cassette
{
    public static async Task Main(string[] args)
    {
        var host = CreateHostBuilder(args)
            .UseConsoleLifetime()
            .Build();

        var services = host.Services;
        var log = services.GetRequiredService<ILogger<Cassette>>();

        var slashService = services.GetRequiredService<SlashService>();
        var updateSlash = await slashService.UpdateSlashCommandsAsync();
        if (!updateSlash.IsSuccess)
        {
            log.LogWarning("Failed to update slash commands: {Reason}", updateSlash.Error.Message);
        }

        await host.RunAsync();
    }

    private static IHostBuilder CreateHostBuilder(string[] args)
    {
        return Host.CreateDefaultBuilder(args)
            .AddDiscordService
            (
                services =>
                {
                    var configuration = services.GetRequiredService<IConfiguration>();

                    return configuration.GetValue<string?>("BOT_TOKEN") ??
                           throw new InvalidOperationException
                           (
                               "No bot token has been provided. " +
                               "Set the BOT_TOKEN environment variable to a valid token."
                           );
                }
            )
            .ConfigureServices
            (
                (_, services) =>
                {
                    services.Configure<DiscordGatewayClientOptions>(g =>
                    {
                        g.Intents
                            |= GatewayIntents.MessageContents
                               | GatewayIntents.GuildVoiceStates
                               | GatewayIntents.Guilds;
                    });
                    services
                        .AddDiscordCommands(true, false)
                        .AddLavalink()
                        .AddCommandTree()
                        .WithCommandGroup<BotCommandGroup>()
                        .WithCommandGroup<ControlsCommandGroup>()
                        .WithCommandGroup<InfoCommandGroup>()
                        .WithCommandGroup<TrustedCommandGroup>();
                }
            )
            .ConfigureLogging
            (
                c => c
                    .AddConsole()
                    .AddFilter("System.Net.Http.HttpClient.*.LogicalHandler", LogLevel.Warning)
                    .AddFilter("System.Net.Http.HttpClient.*.ClientHandler", LogLevel.Warning)
            );
    }
}
