﻿using System.Net;
using System.Text;
using Lavalink4NET.Tracks;
using Remora.Discord.Extensions.Formatting;

namespace Cassette.Extensions;

public static class LavalinkTrackExtensions
{
    public static string Display(this LavalinkTrack track, bool detailed = false)
    {
        var builder = new StringBuilder();

        var titleAppend = track.Title;
        if (track is { Title: "Unknown title", Uri: not null, Provider: StreamProvider.Http })
        {
            titleAppend = WebUtility.UrlDecode(track.Uri.Segments.Last());
        }

        builder.Append(Markdown.Bold(Markdown.Sanitize(titleAppend)));

        if (!detailed)
        {
            return builder.ToString();
        }

        if (track.Author is not "Unknown artist")
        {
            builder.Append(" by ").Append(Markdown.Sanitize(track.Author));
        }

        var duration = track.Duration;
        builder.Append($" ({duration.ReadableDuration()})");

        return builder.ToString();
    }
}
