﻿using Cassette.Extensions;
using Lavalink4NET;
using Lavalink4NET.Players;
using Lavalink4NET.Players.Queued;
using Lavalink4NET.Remora.Discord;
using Remora.Discord.Commands.Contexts;
using Remora.Discord.Commands.Feedback.Services;

namespace Cassette;

public abstract class LavalinkPlayer
{
    public static async ValueTask<QueuedLavalinkPlayer?> GetPlayerAsync(
        ICommandContext commandContext, IAudioService audioService,
        FeedbackService feedbackService, bool connectToVoiceChannel = false)
    {
        var retrieveOptions = new PlayerRetrieveOptions(
            ChannelBehavior: connectToVoiceChannel ? PlayerChannelBehavior.Join : PlayerChannelBehavior.None);

        var result = await audioService.Players
            .RetrieveAsync(commandContext, PlayerFactory.Queued, retrieveOptions);

        if (result.IsSuccess)
        {
            return result.Player;
        }

        var errorMessage = result.Status switch
        {
            PlayerRetrieveStatus.UserNotInVoiceChannel => "You are not connected to a voice channel.",
            PlayerRetrieveStatus.BotNotConnected => "The bot is currently not connected to a voice channel.",
            _ => "Unknown error."
        };

        await feedbackService.SendContextualMessageResult(errorMessage,
            feedbackService.Theme.FaultOrDanger);

        return null;
    }
}
